require 'spec_helper'

describe CloudFormer::Template do
  let(:template) { CloudFormer::Template.new }
  let(:parameter1) { CloudFormer::StringParameter.new('sweet') }
  let(:resource) do
    CloudFormer::IAM::AccessKey.new('KEY') do
      status 'Active'
      user_name 'Aubrey'
    end
  end
  let(:simple_equals_func) do
    CloudFormer::Functions::Equals.new(1, 2)
  end
  let(:param_equals_func) do
    CloudFormer::Functions::Equals.new(parameter1, 2)
  end
  let(:simple_equals) do
    CloudFormer::Condition.new('TestEquals', simple_equals_func)
  end
  let(:param_equals) do
    CloudFormer::Condition.new('TestEquals', param_equals_func)
  end

  describe 'equals' do
    it 'should include parameters' do
      template.add_condition(simple_equals)
      expect(template.dump_json).to eq(
        'AWSTemplateFormatVersion' => '2010-09-09',
        "Conditions" => {
          "TestEquals" => {
            "Fn::Equals" => ["1", "2"],
          },
        },
      )
    end

    it 'should include slightly mode complex parameters' do
      template.add_parameter(parameter1)
      template.add_condition(param_equals)
      expect(template.dump_json).to eq(
        'AWSTemplateFormatVersion' => '2010-09-09',
        "Parameters" => {
          "sweet" => { "Type" => "String" },
        },
        "Conditions" => {
          "TestEquals" => {
            "Fn::Equals" => [{ "Ref" => "sweet" }, "2"],
          },
        },
      )
    end
  end

  describe 'if' do
    let(:simple_if_func) do
      CloudFormer::Functions::If.new(simple_equals, 1, 2)
    end
    let(:simple_if) do
      CloudFormer::Condition.new('TestIf', simple_if_func)
    end

    it 'should work with simple values' do
      template.add_condition(simple_equals)
      template.add_condition(simple_if)
      expect(template.dump_json).to eq(
        "AWSTemplateFormatVersion" => "2010-09-09",
        "Conditions" => {
          "TestEquals" => { "Fn::Equals" => ["1", "2"] },
          "TestIf" => { "Fn::If" => ["TestEquals", "1", "2"] },
        },
      )
    end
  end

  describe 'and' do
    let(:simple_and_func) do
      CloudFormer::Functions::And.new(simple_equals_func, param_equals)
    end
    let(:simple_and) do
      CloudFormer::Condition.new('TestAnd', simple_and_func)
    end

    it 'should work' do
      template.add_condition(param_equals)
      template.add_condition(simple_and)
      expect(template.dump_json).to eq(
        "AWSTemplateFormatVersion" => "2010-09-09",
        "Conditions" => {
          "TestEquals" => { "Fn::Equals" => [{ "Ref" => "sweet" }, "2"] },
          "TestAnd" => { "Fn::And" => [{ "Fn::Equals" => ["1", "2"] }, { "Condition" => "TestEquals" }] },
        },
      )
    end
  end

  describe 'not' do
    let(:simple_not_func) do
      CloudFormer::Functions::Not.new(simple_equals_func)
    end
    let(:simple_not) do
      CloudFormer::Condition.new('TestNot', simple_not_func)
    end

    it 'should work' do
      template.add_condition(simple_not)
      expect(template.dump_json).to eq(
        "AWSTemplateFormatVersion" => "2010-09-09",
        "Conditions" => {
          "TestNot" => { "Fn::Not" => [{ "Fn::Equals" => ["1", "2"] }] },
        },
      )
    end
  end
end
