require 'spec_helper'

describe 'stupid metadata' do

  let(:resource) do
    CloudFormer::AutoScaling::LaunchConfiguration.new('LaunchConfig') do
      key_name 'blahblah'
    end
  end

  it 'should produce JSON' do
    expect(resource.dump_json).to eq({
      'Type' => 'AWS::AutoScaling::LaunchConfiguration',
      'Properties' => {
        'KeyName' => 'blahblah'
      }
    })
  end

  describe 'with some authentication metadata' do

    let(:authentication_item) do
      CloudFormer::CloudFormation::AuthenticationItem.new do
        access_key_id 12
        password 'ack'
        secret_key 'boom'
      end
    end

    let(:authentication) do
      item = authentication_item
      CloudFormer::CloudFormation::Authentication.new do
        items [item]
      end
    end

    it 'should produce json for the item' do
      expect(authentication_item.dump_json).to eq({
        'accessKeyId' => 12,
        'password' => 'ack',
        'secretKey' => 'boom'
      })
    end

    it 'should let me add the metadata to the resource' do
      resource.add_metadata(authentication)
    end

    it 'should put the metadata in the json' do
      resource.add_metadata(authentication)
      expect(resource.dump_json).to eq({
        'Type' => 'AWS::AutoScaling::LaunchConfiguration',
        'Properties' => {
          'KeyName' => 'blahblah'
        },
        'Metadata' => {
          'AWS::CloudFormation::Authentication' => {
            authentication_item.get_name => {
              'accessKeyId' => 12,
              'password' => 'ack',
              'secretKey' => 'boom'
            }
          }
        }
      })
    end
  end

  describe 'with some init metadata' do

    let(:command) do
      CloudFormer::CloudFormation::ConfigSetCommand.new do
        name 'see what is in tmp'
        command 'ls /tmp'
        cwd '/root'
      end
    end

    let(:file) do
      CloudFormer::CloudFormation::ConfigSetFile.new do
        name '/tmp/ack'
        source 'hahahahaha'
        mode '000666'
      end
    end

    let(:config_set_item) do
      cmd = command
      f = file
      CloudFormer::CloudFormation::ConfigSetItem.new do
        name 'first'
        commands [cmd]
        files [f]
      end
    end

    let(:config_set) do
      item = config_set_item
      CloudFormer::CloudFormation::ConfigSet.new do
        name 'badass'
        items [item]
      end
    end

    let(:init) do
      set = config_set
      CloudFormer::CloudFormation::Init.new do
        config_sets [set]
      end
    end

    it 'should produce json for the item' do
      expect(init.dump_json).to eq({
        'configSets' => { 'badass' => ['first'] },
        "first" => {
          "commands" => {
            "see what is in tmp" => {"command"=>"ls /tmp", "cwd"=>"/root"}
          },
          "files" => {
            "/tmp/ack"=>{"source"=>"hahahahaha", "mode"=>"000666"}
          }
        }
      })
    end

    it 'should put the metadata in the json' do
      resource.add_metadata(init)
      expect(resource.dump_json).to eq({
        'Type' => 'AWS::AutoScaling::LaunchConfiguration',
        'Properties' => {
          'KeyName' => 'blahblah'
        },
        'Metadata' => {
          'AWS::CloudFormation::Init' => {
            "configSets" => {"badass"=>["first"]},
            "first" => {
              "commands" => {
                "see what is in tmp" => {"command"=>"ls /tmp", "cwd"=>"/root"}
              },
              "files" => {
                "/tmp/ack"=>{"source"=>"hahahahaha", "mode"=>"000666"}
              }
            }
          }
        }
      })
    end
  end

end
