require 'spec_helper'

describe CloudFormer::IAM::User do

  describe 'with a basic user' do
    let(:group1) do
      CloudFormer::IAM::Group.new('group1') do
        path 'ack'
      end
    end

    let(:group2) do
      CloudFormer::IAM::Group.new('group2') do
        path 'acky'
      end
    end

    let(:user) do
      g1 = group1
      g2 = group2
      CloudFormer::IAM::User.new('USER') do
        path 'a/b/c'
        groups [g1, g2]
        login_profile({ 'Password' => 'ack' })
      end
    end

    it 'should let me get the status' do
      expect(user.path).to eq('a/b/c')
    end

    it 'should make some awesome json' do
      expect(user.dump_json).to eq({
        'Type' => 'AWS::IAM::User',
        'Properties' => {
          'Path' => 'a/b/c',
          'Groups' => [
            { 'Ref' => 'group1' },
            { 'Ref' => 'group2' }
          ],
          'LoginProfile' => {
            'Password' => 'ack'
          }
        }
      })
    end
  end

  describe 'with some policies' do

    let(:group1) do
      CloudFormer::IAM::Group.new('group1') do
        path 'ack'
      end
    end

    let(:group2) do
      CloudFormer::IAM::Group.new('group2') do
        path 'acky'
      end
    end

    let(:policy1) do
      CloudFormer::IAM::Policy.new('POLICY') do
        policy_name 'root'
        policy_document({
          'Statement' => [{
            'Effect' => 'Allow',
            'Action' => 'cloudformation:DescribeStackResource',
            'Resource' => '*'
          }]
        })
      end
    end

    let(:policy2) do
      CloudFormer::IAM::Policy.new('POLICY') do
        policy_name 'root'
        policy_document({
          'Statement' => [{
            'Effect' => 'DISALLOW',
            'Action' => 'cloudformation:DescribeStackResource',
            'Resource' => '*'
          }]
        })
      end
    end

    let(:user) do
      p1 = policy1
      p2 = policy2
      g1 = group1
      g2 = group2
      CloudFormer::IAM::User.new('USER') do
        path 'a/b/c'
        groups [g1, g2]
        login_profile({ 'Password' => 'ack' })
        policies [p1, p2]
      end
    end

    it 'should make some awesome json' do
      expect(user.dump_json).to eq({
        'Type' => 'AWS::IAM::User',
        'Properties' => {
          'Path' => 'a/b/c',
          'Groups' => [
            { 'Ref' => 'group1' },
            { 'Ref' => 'group2' }
          ],
          'LoginProfile' => {
            'Password' => 'ack'
          },
          'Policies' => [
            {
              "PolicyName"=>"root",
              "PolicyDocument"=>{
                "Statement"=>[{
                  "Effect"=>"Allow",
                  "Action"=>"cloudformation:DescribeStackResource",
                  "Resource"=>"*"
                }]
              }
            },
            {
              "PolicyName"=>"root",
              "PolicyDocument"=>{
                "Statement"=>[{
                  "Effect"=>"DISALLOW",
                  "Action"=>"cloudformation:DescribeStackResource",
                  "Resource"=>"*"
                }] 
              }
            }
          ]
        }
      })
    end
  end

end
